local module = {}
module.name = "Chat"
module.Init = function()
	if not SCDB.modules[module.name] then return end
	SCDB[module.name] = SCDB[module.name] or {}
	if SCDB[module.name]["CopyURL"] == nil then SCDB[module.name]["CopyURL"] = true end
	if SCDB[module.name]["CopyChat"] == nil then SCDB[module.name]["CopyChat"] = true end
	if SCDB[module.name]["ChatScroll"] == nil then SCDB[module.name]["ChatScroll"] = true end
	if SCDB[module.name]["MoveEditBox"] == nil then SCDB[module.name]["MoveEditBox"] = true end
	if SCDB[module.name]["Use custom font"] == nil then SCDB[module.name]["Use custom font"] = true end
	local opts = SCDB[module.name]
	
	local replaceschan = {
		[CHAT_MSG_GUILD] = '[G]',
		[CHAT_MSG_PARTY] = '[P]',
		[CHAT_MSG_RAID] = '[R]',
		[CHAT_MSG_RAID_LEADER] = '[LR]',
		[CHAT_MSG_RAID_WARNING] = '[RW]',
		[CHAT_MSG_OFFICER] = '[O]',
		[CHAT_MSG_BATTLEGROUND] = '[BG]',
		[CHAT_MSG_BATTLEGROUND_LEADER] = '[BGL]',
		['(%d+)%. .-'] = '[%1]',
	}
	
	ToggleChatColorNamesByClassGroup(true, "SAY")
	ToggleChatColorNamesByClassGroup(true, "EMOTE")
	ToggleChatColorNamesByClassGroup(true, "YELL")
	ToggleChatColorNamesByClassGroup(true, "GUILD")
	ToggleChatColorNamesByClassGroup(true, "OFFICER")
	ToggleChatColorNamesByClassGroup(true, "GUILD_ACHIEVEMENT")
	ToggleChatColorNamesByClassGroup(true, "ACHIEVEMENT")
	ToggleChatColorNamesByClassGroup(true, "WHISPER")
	ToggleChatColorNamesByClassGroup(true, "PARTY")
	ToggleChatColorNamesByClassGroup(true, "PARTY_LEADER")
	ToggleChatColorNamesByClassGroup(true, "RAID")
	ToggleChatColorNamesByClassGroup(true, "RAID_LEADER")
	ToggleChatColorNamesByClassGroup(true, "RAID_WARNING")
	ToggleChatColorNamesByClassGroup(true, "BATTLEGROUND")
	ToggleChatColorNamesByClassGroup(true, "BATTLEGROUND_LEADER")
	ToggleChatColorNamesByClassGroup(true, "CHANNEL1")
	ToggleChatColorNamesByClassGroup(true, "CHANNEL2")
	ToggleChatColorNamesByClassGroup(true, "CHANNEL3")
	ToggleChatColorNamesByClassGroup(true, "CHANNEL4")
	ToggleChatColorNamesByClassGroup(true, "CHANNEL5")

	------------------------------------------------------------------------
	-- Enhance/rewrite a Blizzard feature, chatframe mousewheel.
	------------------------------------------------------------------------

	local ScrollLines = 3 -- set the jump when a scroll is done !
	function FloatingChatFrame_OnMouseScroll(self, delta)
		if delta < 0 then
			if IsShiftKeyDown() then
				self:ScrollToBottom()
			else
				for i = 1, ScrollLines do
					self:ScrollDown()
				end
			end
		elseif delta > 0 then
			if IsShiftKeyDown() then
				self:ScrollToTop()
			else
				for i = 1, ScrollLines do
					self:ScrollUp()
				end
			end
		end
	end

	-----------------------------------------------------------------------------
	-- copy url
	-----------------------------------------------------------------------------
	if opts.CopyURL == true then
		local color = "0022FF"
		local pattern = "[wWhH][wWtT][wWtT][\46pP]%S+[^%p%s]"

		function string.color(text, color)
			return "|cff"..color..text.."|r"
		end

		function string.link(text, type, value, color)
			return "|H"..type..":"..tostring(value).."|h"..tostring(text):color(color or "ffffff").."|h"
		end

		StaticPopupDialogs["LINKME"] = {
			text = "URL COPY",
			button2 = CANCEL,
			hasEditBox = true,
			hasWideEditBox = true,
			timeout = 0,
			exclusive = 1,
			hideOnEscape = 1,
			EditBoxOnEscapePressed = function(self) self:GetParent():Hide() end,
			whileDead = 1,
			maxLetters = 255,
		}

		local function f(url)
			return string.link("["..url.."]", "url", url, color)
		end

		local function hook(self, text, ...)
			self:f(text:gsub(pattern, f), ...)
		end

		for i = 1, NUM_CHAT_WINDOWS do
			if ( i ~= 2 ) then
				local frame = _G["ChatFrame"..i]
				frame.f = frame.AddMessage
				frame.AddMessage = hook
			end
		end

		local f = ChatFrame_OnHyperlinkShow
		function ChatFrame_OnHyperlinkShow(self, link, text, button)
			local type, value = link:match("(%a+):(.+)")
			if ( type == "url" ) then
				local sname, stype, svalue = link:match("(%a+).com\/(%a+)=(%d+)")
				if sname and sname:find("wowhead") then
					if stype == "item" then
						local _,slink = GetItemInfo(svalue)
						if slink then
							ShowUIPanel(ItemRefTooltip)
							ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
							ItemRefTooltip:SetHyperlink(slink)
						end
					elseif stype == "spell" then
						local slink = GetSpellLink(svalue)
						if slink then
							ShowUIPanel(ItemRefTooltip)
							ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
							ItemRefTooltip:SetHyperlink(slink)
						end
					end
				else
					local dialog = StaticPopup_Show("LINKME")
					local editbox = _G[dialog:GetName().."EditBox"]  
					editbox:SetText(value)
					editbox:SetFocus()
					editbox:HighlightText()
					local button = _G[dialog:GetName().."Button2"]
					button:ClearAllPoints()
					button:SetPoint("CENTER", editbox, "CENTER", 0, -30)
				end
			else
				f(self, link, text, button)
			end
		end
	end
	
	-----------------------------------------------------------------------------
	-- Copy Chat (credit: shestak for this version)
	-----------------------------------------------------------------------------
	if opts.CopyChat == true then
		local lines = {}
		local frame = nil
		local editBox = nil
		local isf = nil
		
		local function MakeMovable(f, ...)
			f:EnableMouse(true)
			f:RegisterForDrag("LeftButton")
			f:SetClampedToScreen(true)
			f:SetMovable(true)
			f:SetScript("OnDragStart", function(self) self:StartMoving() end)
			f:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
		end
		
		local function CreatCopyFrame()
			copyframe = _G["CopyFrame"] or CreateFrame("Frame", "CopyFrame", UIParent)
			copyframe:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tileSize = 16, edgeSize = 16, insets = {left=3, right=3, top=3, bottom=3}, })
			copyframe:SetBackdropColor(.01, .01, .01, .85)
			copyframe:SetBackdropBorderColor(.4, .4, .4, 1)
			copyframe:SetWidth(410)
			copyframe:SetHeight(200)
			copyframe:SetScale(1)
			copyframe:SetPoint("CENTER")
			copyframe:Hide()
			copyframe:SetFrameStrata("DIALOG")

			local scrollArea = _G["CopyScroll"] or CreateFrame("ScrollFrame", "CopyScroll", copyframe, "UIPanelScrollFrameTemplate")
			scrollArea:SetPoint("TOPLEFT", copyframe, "TOPLEFT", 8, -30)
			scrollArea:SetPoint("BOTTOMRIGHT", copyframe, "BOTTOMRIGHT", -30, 8)

			editBox = _G["CopyBox"] or CreateFrame("EditBox", "CopyBox", copyframe)
			editBox:SetMultiLine(true)
			editBox:SetMaxLetters(99999)
			editBox:EnableMouse(true)
			editBox:SetAutoFocus(false)
			editBox:SetFontObject("ChatFontNormal")
			editBox:SetWidth(410)
			editBox:SetHeight(200)
			editBox:SetScript("OnEscapePressed", function() copyframe:Hide() end)

			scrollArea:SetScrollChild(editBox)

			local close = _G["CopyCloseButton"] or CreateFrame("Button", "CopyCloseButton", copyframe, "UIPanelCloseButton")
			close:SetPoint("TOPRIGHT", copyframe, "TOPRIGHT")

			isf = true
			return copyframe
		end

		local function GetLines(...)
			--[[ Grab all those lines ]]--
			local ct = 1
			for i = select("#", ...), 1, -1 do
				local region = select(i, ...)
				if region:GetObjectType() == "FontString" then
					lines[ct] = tostring(region:GetText())
					ct = ct + 1
				end
			end
			return ct - 1
		end

		local function Copy(cf)
			local _, size = cf:GetFont()
			FCF_SetChatWindowFontSize(cf, cf, 0.01)
			local lineCt = GetLines(cf:GetRegions())
			local text = table.concat(lines, "\n", 1, lineCt)
			FCF_SetChatWindowFontSize(cf, cf, size)
			if not isf then 
				MakeMovable(CreatCopyFrame())
			end
			copyframe:Show()
			editBox:SetText(text)
			editBox:HighlightText(0)
		end

		for i = 1, NUM_CHAT_WINDOWS do
			if i ~= 2 then
				local cf = _G[format("ChatFrame%d",  i)]
				local but = CreateFrame("button", "copybutton_"..i, cf)
				but:SetNormalTexture("Interface\\Buttons\\UI-Panel-SmallerButton-Up")
				but:SetPushedTexture("Interface\\Buttons\\UI-Panel-SmallerButton-Down")
				but:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
				but:SetPoint("TOPRIGHT", 0, 0)
				but:SetHeight(32)
				but:SetWidth(32)
				but:SetAlpha(0)
				but:SetScript("OnClick", function(self) Copy(self:GetParent()) end)
				but:SetScript("OnEnter", function(self) self:SetAlpha(1) end)
				but:SetScript("OnLeave", function(self) self:SetAlpha(0) end)
			end
		end
	end

	-- Player entering the world
	local eventframe = CreateFrame("Frame")
	eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
	eventframe:SetScript("OnEvent", function(self, event)
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		
		-- Chat EditBox animation by AlleyKat
		local edit = CreateFrame("frame", nil, UIParent)
			edit:SetPoint("TOPLEFT",ChatFrame1EditBox,8,-4)
			edit:SetPoint("BOTTOMRIGHT",ChatFrame1EditBox,-5,4)
			edit:SetFrameLevel(42)
			edit:SetFrameStrata(ChatFrame1EditBox:GetFrameStrata())
			
		ChatFrame1EditBox:SetFrameLevel(40)
			
		local a_high = 119
		local show_in = function(self,nf)
			self.show_in = self:CreateAnimationGroup("In")
			self.show_in.a = self.show_in:CreateAnimation("Translation")
			self.show_in.a:SetOffset(0,-a_high - 80)
			self.show_in.a:SetDuration(0)
			self.show_in.a:SetOrder(1)
			self.show_in.b = self.show_in:CreateAnimation("Translation")
			self.show_in.b:SetOffset(0,a_high + 80)
			self.show_in.b:SetDuration(.2)
			self.show_in.b:SetOrder(2)
			self.show_in.b:SetSmoothing("OUT")
		end

		local show_out = function(self,nf)
			self.show_out = self:CreateAnimationGroup("OUT")
			self.show_out.a = self.show_out:CreateAnimation("Translation")
			self.show_out.a:SetOffset(0,-a_high - 80)
			self.show_out.a:SetDuration(.2)
			self.show_out.a:SetOrder(1)
			self.show_out.a:SetSmoothing("IN")
		end

		show_in(ChatFrame1EditBox)
		show_in(edit)
		show_out(edit)

		edit.show_out:SetScript("OnFinished",function() edit:Hide() end)

		ChatFrame1EditBox:HookScript("OnShow", function(self)
			edit:Show()
			edit.show_out:Stop()
			edit.show_in:Play()
			self.show_in:Play()
		end)

		ChatFrame1EditBox:HookScript("OnHide", function(self)
			edit.show_in:Stop()
			self.show_in:Stop()
			edit.show_out:Play()
		end)

		-----------------------------------------

		if opts.ChatScroll == true then
			SetCVar("chatMouseScroll", 1)
		else
			SetCVar("chatMouseScroll", 0)
		end

		-- Hook into the AddMessage function
		local function AddMessageHook(frame, text, ...)
			-- chan text smaller or hidden
			for k,v in pairs(replaceschan) do
				text = string.gsub(text, '|h%['..k..'%]|h', '|h'..v..'|h')
			end
			text = string.gsub(text, "|Hplayer:%s|h[%s]:|h", "|Hplayer:%1|h%2:|h")
			return frame.AddMessageOriginal(frame, text, ...)
		end

		for i = 1, NUM_CHAT_WINDOWS do
			if ( i ~= 2 ) then
			local frame = _G["ChatFrame"..i]
				frame.AddMessageOriginal = frame.AddMessage
				frame.AddMessage = AddMessageHook
			end
		end

		ChatFrameMenuButton:Hide()
		ChatFrameMenuButton:SetScript("OnShow", function(self) self:Hide() end)

		-- Hide friends micro button (added in 3.3.5)
		FriendsMicroButton:SetScript("OnShow", FriendsMicroButton.Hide)
		FriendsMicroButton:Hide()

		GeneralDockManagerOverflowButton:SetScript("OnShow", GeneralDockManagerOverflowButton.Hide)
		GeneralDockManagerOverflowButton:Hide()
		
		hooksecurefunc("ChatEdit_OnTextSet", function(text, chatFrame)
			if ( not chatFrame ) then
				chatFrame = DEFAULT_CHAT_FRAME;
			end
			local x=({chatFrame.editBox:GetRegions()})
			local r,g,b,a = x[9]:GetVertexColor()
		end)
		
		hooksecurefunc("ChatEdit_OnEscapePressed", function(eb)
			eb:Hide()
		end)
		
		-- Remember last channel
		ChatTypeInfo.WHISPER.sticky = 1
		ChatTypeInfo.BN_WHISPER.sticky = 1
		ChatTypeInfo.OFFICER.sticky = 1
		ChatTypeInfo.RAID_WARNING.sticky = 1
		ChatTypeInfo.CHANNEL.sticky = 1
		
		-----------------------------------------------------------------------
		-- OVERWRITE GLOBAL VAR FROM BLIZZARD
		-----------------------------------------------------------------------

		CHAT_FRAME_FADE_OUT_TIME = 0
		CHAT_TAB_HIDE_DELAY = 0
		CHAT_FRAME_TAB_SELECTED_MOUSEOVER_ALPHA = 1
		CHAT_FRAME_TAB_SELECTED_NOMOUSE_ALPHA = 0
		CHAT_FRAME_TAB_NORMAL_MOUSEOVER_ALPHA = 1
		CHAT_FRAME_TAB_NORMAL_NOMOUSE_ALPHA = 0
		CHAT_FRAME_TAB_ALERTING_MOUSEOVER_ALPHA = 1
		CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = 0

		COPPER_AMOUNT = "%d|cFF954F28"..COPPER_AMOUNT_SYMBOL.."|r"
		SILVER_AMOUNT = "%d|cFFC0C0C0"..SILVER_AMOUNT_SYMBOL.."|r"
		GOLD_AMOUNT = "%d|cFFF0D440"..GOLD_AMOUNT_SYMBOL.."|r"	
	
		YOU_LOOT_MONEY = "+%s"
		YOU_LOOT_MONEY_GUILD = "+%s (%s)"
		LOOT_MONEY_SPLIT = "+%s."
		LOOT_MONEY_SPLIT_GUILD = "+%s. (%s)"
		
		COMBATLOG_XPGAIN_EXHAUSTION1 = ERR_PLAYER_DIED_S.." "..COMBAT_XP_GAIN:lower().." +%d (%s "..COMBAT_XP_GAIN:lower().." %s+)"
		COMBATLOG_XPGAIN_EXHAUSTION2 = ERR_PLAYER_DIED_S.." "..COMBAT_XP_GAIN:lower().." +%d (%s "..COMBAT_XP_GAIN:lower().." %s+)"
		COMBATLOG_XPGAIN_EXHAUSTION4 = ERR_PLAYER_DIED_S.." "..COMBAT_XP_GAIN:lower().." +%d (%s "..COMBAT_XP_GAIN:lower().." %s-)"
		COMBATLOG_XPGAIN_EXHAUSTION5 = ERR_PLAYER_DIED_S.." "..COMBAT_XP_GAIN:lower().." +%d (%s "..COMBAT_XP_GAIN:lower().." %s-)"
		COMBATLOG_XPGAIN_FIRSTPERSON = ERR_PLAYER_DIED_S.." "..COMBAT_XP_GAIN:lower().." +%d"
		
		
		for i = 1, NUM_CHAT_WINDOWS do
			_G["ChatFrame"..i]:SetClampRectInsets(0,0,0,0)
			
			for j = 1, #CHAT_FRAME_TEXTURES do
				_G["ChatFrame"..i..CHAT_FRAME_TEXTURES[j]]:SetTexture(nil)
			end
			
			-- Hide chat buttons
			_G["ChatFrame"..i.."ButtonFrameUpButton"]:Hide()
			_G["ChatFrame"..i.."ButtonFrameDownButton"]:Hide()
			_G["ChatFrame"..i.."ButtonFrameBottomButton"]:Hide()
			_G["ChatFrame"..i.."ButtonFrameMinimizeButton"]:Hide()
			_G["ChatFrame"..i.."ButtonFrame"]:Hide()

			_G["ChatFrame"..i.."ButtonFrameUpButton"]:SetScript("OnShow", function(self) self:Hide() end)
			_G["ChatFrame"..i.."ButtonFrameDownButton"]:SetScript("OnShow", function(self) self:Hide() end)
			_G["ChatFrame"..i.."ButtonFrameBottomButton"]:SetScript("OnShow", function(self) self:Hide() end)
			_G["ChatFrame"..i.."ButtonFrameMinimizeButton"]:SetScript("OnShow", function(self) self:Hide() end)
			_G["ChatFrame"..i.."ButtonFrame"]:SetScript("OnShow", function(self) self:Hide() end)


			-- Stop the chat frame from fading out
			--_G["ChatFrame"..i]:SetFading(false)
			
			-- Texture and align the chat edit box
			local editbox = _G["ChatFrame"..i.."EditBox"]
			local left, mid, right = select(6, editbox:GetRegions()); left:Hide(); mid:Hide(); right:Hide()
			editbox:ClearAllPoints()
			if opts.MoveEditBox == true then
				editbox:SetPoint("BOTTOMLEFT", ChatFrame1, "TOPLEFT", 0, 30)
				editbox:SetPoint("TOPRIGHT", ChatFrame1, "TOPRIGHT", 0, 60)
			else
				editbox:SetPoint("TOPLEFT", ChatFrame1, "BOTTOMLEFT", 0, 0)
				editbox:SetPoint("BOTTOMRIGHT", ChatFrame1, "BOTTOMRIGHT", 0, -30)
			end
			editbox:Hide()
			
			-- Disable alt key usage
			editbox:SetAltArrowKeyMode(false)
			
			local font, size, ss = _G[format("ChatFrame%d", i)]:GetFont()
			if SCDB[module.name]["Use custom font"] then
				_G[format("ChatFrame%d", i)]:SetFont(STANDARD_TEXT_FONT, size, "OUTLINE")
			else
				_G[format("ChatFrame%d", i)]:SetFont(font, size, "OUTLINE")
			end
			_G[format("ChatFrame%d", i)]:SetShadowOffset(0, 0)
			--_G[format("ChatFrame%d", i)]:SetMinResize(50,50)
			--_G[format("ChatFrame%d", i)]:SetClampedToScreen(false)
						
			--FCF_SavePositionAndDimensions(_G[format("ChatFrame%d", i)])
		end

	end)
	
 end
 tinsert(SuperClassic.modules, module) -- finish him!